#include <qapplication.h>
#include <qmessagebox.h>
#include <qhostaddress.h>
#include <qlineedit.h>
#include <qspinbox.h>
#include <qsocket.h>
#include <qtextedit.h>
#include <qtextstream.h>
#include <qcheckbox.h>
#include <qlistbox.h>
#include <qdir.h>
#include <qfileinfo.h>

#include "blinksharemainwindow.h"
#include "blinkshareconfig.h"
#include "global.h"

/* 
 *  Constructs a BlinkShareMain which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 */
BlinkShareMain::BlinkShareMain( QWidget* parent,  const char* name, WFlags fl )
    : BlinkShareMainBase( parent, name, fl )
{
    m_config = new BlinkShareConfig(this);
    m_socket = 0;
    emit signalConnected(false);
    slotLocalList();
    slotLoveList();
}

/*  
 *  Destroys the object and frees any allocated resources
 */
BlinkShareMain::~BlinkShareMain()
{
}

/*
 * public slot
 */
void BlinkShareMain::slotQuit()
{
    int ret = QMessageBox::warning(this, "Close BlinkShare?",
            "Closing BlinkShare will close the connection!\n"
            "\n"
            "Are you sure you want to close?",
            "Yes",
            "No",
           QString::null,
           1, 1);
    if(ret == 0) {
        qApp->quit();
    }

}

/*
 * public slot
 */
void BlinkShareMain::closeEvent(QCloseEvent* event)
{
    event->ignore();
    this->slotQuit();
}


/*
 * public slot
 */
void BlinkShareMain::slotConfigure()
{
    if(m_socket != 0) {
        delete m_socket;
        m_socket = 0;
    }
    emit signalConnected(false);
    if(m_config->exec()) {
        m_socket = new QSocket(this);
        m_socket->connectToHost(m_config->HostnameEdit->text(),
                m_config->PortEdit->value());
        connect(m_socket, SIGNAL(error(int)),
                this, SLOT(slotError(int)));
        connect(m_socket, SIGNAL(connected()),
                this, SLOT(slotConnected()));
        connect(m_socket, SIGNAL(connectionClosed()),
                this, SLOT(slotConnectionClosed()));
        connect(m_socket, SIGNAL(delayedCloseFinished()),
                this, SLOT(slotDelayedCloseFinished()));
        connect(m_socket, SIGNAL(readyRead()),
                this, SLOT(slotReadReady()));
        c_list = false;
        c_info = false;
        c_send = false;
        c_retr = false;
        c_love = false;
        c_play = false;
        c_dele = false;
        m_state = StateConnect;
        ServerMotto->clear();
    } else {
        qDebug("Rejected");
    }
}

/*
 * public slot
 */
void BlinkShareMain::slotReadReady() {
    QString line;
    QString error;
    QTextStream ts(m_socket);
    while (m_socket->canReadLine()) {
        line = ts.readLine();
        /* ------------------------------------------------
         * CONNECT-Response (Welcome-Message + MOTTO)
         * ------------------------------------------------ */
        if(m_state == StateConnect) {
            if(line[0] == '+') {
                continue;
            }
            if(line == ".") {
                ts << "HELO\n";
                m_state = StateHelo;
                continue;
            }
            if(line == "") {
                line = " ";
            }
            ServerMotto->append(line);
            continue;
        }

        /* ------------------------------------------------
         * Response to the HELO command
         * ------------------------------------------------ */
        if(m_state == StateHelo) {
            if(line[0] == '+') {
                continue;
            }
            if(line == ".") {
                if(m_config->AnonymousCheckBox->isChecked()) {
                    ts << "USER anonymous\n";
                } else {
                    ts << "USER " << m_config->UserEdit->text() << "\n";
                }
                m_state = StateUser;
                continue;
            }
            if(line == "COMD LIST") {
                c_list = true;
            } else if(line == "COMD INFO") {
                c_info = true;
            } else if(line == "COMD RETR") {
                c_retr = true;
            } else if(line == "COMD SEND") {
                c_send = true;
            } else if(line == "COMD LOVE") {
                c_love = true;
            } else if(line == "COMD PLAY") {
                c_play = true;
            } else if(line == "COMD DELE") {
                c_dele = true;
            }
        }

        /* ------------------------------------------------
         * Response to the USER command
         * ------------------------------------------------ */
        if(m_state == StateUser) {
            if(line[0] == '-') {
                delete m_socket;
                m_socket = 0;
                int ret = QMessageBox::warning(this, "Login failed",
                         "Re-Configure?",
                         "Yes",
                         "No",
                        QString::null,
                        1, 1);
                if(ret == 0) {
                     slotConfigure();
                }
            } else if(line[0] == '+') {
                if(m_config->AnonymousCheckBox->isChecked()) {
                    ts << "PASS anonymous\n";
                } else {
                    ts << "PASS " << m_config->PassEdit->text() << "\n";
                }
                m_state = StatePass;
                continue;
            } else {
                delete m_socket;
                m_socket = 0;
                int ret = QMessageBox::warning(this, "Login failed",
                         "The server issued an unexpected response\n"
                         "\n"
                         "Re-Configure?",
                         "Yes",
                         "No",
                        QString::null,
                        1, 1);
                if(ret == 0) {
                     slotConfigure();
                }
            }
        }

        /* ------------------------------------------------
         * Response to the PASS-command
         * ------------------------------------------------ */
        if(m_state == StatePass) {
            if(line[0] == '-') {
                delete m_socket;
                m_socket = 0;
                int ret = QMessageBox::warning(this, "Login failed",
                         "Re-Configure?",
                         "Yes",
                         "No",
                        QString::null,
                        1, 1);
                if(ret == 0) {
                     slotConfigure();
                }
            } else if(line[0] == '+') {
                m_state = StateNone;
                slotList();
                continue;
            } else {
                delete m_socket;
                m_socket = 0;
                int ret = QMessageBox::warning(this, "Login failed",
                         "The server issued an unexpected response\n"
                         "\n"
                         "Re-Configure?",
                         "Yes",
                         "No",
                        QString::null,
                        1, 1);
                if(ret == 0) {
                     slotConfigure();
                }
            }
        }

        /* ------------------------------------------------
         * Response to the LIST command
         * ------------------------------------------------ */
        if(m_state == StateList) {
            if(line[0] == '-') {
                QMessageBox::information(this, "LIST Command",
                        "The LIST command failed!\n"
                        "\n"
                        "Please check your server-configuration!");
            } else if(line[0] == '+') {
                continue;
            } else if(line == ".") {
                m_state = StateNone;
                emit signalConnected(true);
                continue;
            }
            ServerListBox->insertItem(line);
        }

        /* ------------------------------------------------
         * Response to the INFO command
         * ------------------------------------------------ */
        if(m_state == StateInfo) {
            if(line[0] == '-') {
                QMessageBox::information(this, "INFO Command",
                        "The INFO command failed!");
                continue;
            } else if(line[0] == '+') {
                continue;
            } else if(line == ".") {
                m_state = StateNone;
                emit signalConnected(true);
                continue;
            }
            ServerMotto->append(line);
        }

        /* ------------------------------------------------
         * Response to the DELE command
         * ------------------------------------------------ */
        if(m_state == StateDele) {
            m_state = StateNone;
            emit signalConnected(true);
            if(line[0] == '-') {
                QMessageBox::information(this, "DELE Command",
                        "Could not delete file!");
            } else if(line[0] == '+') {
                slotList();
                continue;
            }
        }

        /* ------------------------------------------------
         * Response to the SEND command
         * ------------------------------------------------ */
        if(m_state == StateSend) {
            m_state = StateSendAccept;
            if(line[0] == '-') {
                QMessageBox::information(this, "SEND Command",
                        "Server does not accept file for upload!");
                m_state = StateNone;
            } else if(line[0] == '+') {
                m_movie->writeTextStream(ts);
                ts << ".\n";
            }
            delete m_movie;
            m_movie=0;
            continue;
        }
        /* ------------------------------------------------
         * Response to the SEND-finished command
         * ------------------------------------------------ */
        if(m_state == StateSendAccept) {
            m_state = StateNone;
            emit signalConnected(true);
            if(line[0] == '-') {
                QMessageBox::information(this, "SEND Command",
                        "Server does not accept file-format!");
            }
            slotList();
        }

        /* ------------------------------------------------
         * Response to the SEND-ALL command
         * ------------------------------------------------ */
        if(m_state == StateSendAll) {
            m_state = StateSendAllAccept;
            if(line[0] == '-') {
                error = QString("File %1 not accepted by server")
                    .arg(m_movie->getName());
                ServerMotto->append(error);
                delete m_movie;
                m_movie=0;
                slotUpAllNext();
            } else if(line[0] == '+') {
                m_movie->writeTextStream(ts);
                ts << ".\n";
            }
        }
        /* ------------------------------------------------
         * Response to the SEND-ALL one-finished command :-)
         * ------------------------------------------------ */
        if(m_state == StateSendAllAccept) {
            if(line[0] == '-') {
                error = QString("File-format of %1 not accepted by server")
                    .arg(m_movie->getName());
            } else {
                error = QString("OK: %1 uploaded")
                    .arg(m_movie->getName());
            }
            ServerMotto->append(error);
            delete m_movie;
            m_movie=0;
            slotUpAllNext();
            continue;
        }

        /* ------------------------------------------------
         * Response to the RETR command
         * ------------------------------------------------ */
        if(m_state == StateRetr) {
            if(line[0] == '-') {
                QMessageBox::information(this, "RETR Command",
                        "The RETR command failed!");
                m_state = StateNone;
                delete m_movie;
                m_movie = 0;
                emit signalConnected(true);
                continue;
            } else if(line[0] == '+') {
                continue;
            } else if(line == ".") {
                m_movie->stopReadStream();
                if(!m_movie->parseFile()) {
                    QMessageBox::information(this, "RETR Command",
                            "Sorry, file does not parse!");
                } else {
                    QString filename = QString(CLNTDIR "movie/share/%1")
                        .arg(m_movie->getName());
                    m_movie->saveFile(filename);
                }
                m_state = StateNone;
                emit signalConnected(true);
                delete m_movie;
                m_movie = 0;
                slotList();
                continue;
            } else {
                m_movie->addLineReadStream(line);
            }
        }

        /* ------------------------------------------------
         * Response to the RETR-ALL command
         * ------------------------------------------------ */
        if(m_state == StateRetrAll) {
            if(line[0] == '-') {
                error = QString("Get for %1 failed!")
                    .arg(m_movie->getName());
                ServerMotto->append(error);
                delete m_movie;
                m_movie = 0;
                slotDownAllNext();
                continue;
            } else if(line[0] == '+') {
                continue;
            } else if(line == ".") {
                m_movie->stopReadStream();
                if(!m_movie->parseFile()) {
                    error = QString("File %1 does not parse!")
                        .arg(m_movie->getName());
                    ServerMotto->append(error);
                } else {
                    QString filename = QString(CLNTDIR "movie/share/%1")
                        .arg(m_movie->getName());
                    m_movie->saveFile(filename);
                    error = QString("OK: %1 saved")
                        .arg(m_movie->getName());
                    ServerMotto->append(error);
                }
                m_state = StateNone;
                delete m_movie;
                m_movie = 0;
                slotDownAllNext();
                continue;
            } else {
                m_movie->addLineReadStream(line);
            }
        }

        /* ------------------------------------------------
         * Response to the LOVE command
         * ------------------------------------------------ */
        if(m_state == StateLove) {
            if(line[0] == '-') {
                QMessageBox::information(this, "LOVE Command",
                        "Server does not accept file for upload!");
                delete m_movie;
                m_movie=0;
                m_state = StateNone;
            } else if(line[0] == '+') {
                m_movie->writeTextStream(ts);
                ts << ".\n";
                m_state = StateLoveAccept;
            }
            continue;
        }
        /* ------------------------------------------------
         * Response to the LOVE-finished command
         * ------------------------------------------------ */
        if(m_state == StateLoveAccept) {
            if(line[0] == '-') {
                QMessageBox::information(this, "LOVE Command",
                        "Server does not accept file-format!");
                emit signalConnected(true);
                delete m_movie;
                m_movie=0;
                m_state = StateNone;
                slotList();
                continue;
            } else if(line[0] == '+') {
                m_lovefirst = true;
                ServerMotto->clear();
                continue;
            } else if(line[0] == '.') {
                delete m_movie;
                m_movie=0;
                m_state = StateNone;
                emit signalConnected(true);
                slotList();
            } else {
                if(m_lovefirst) {
                    m_lovefirst = false;
                    qDebug("##### ->%s<-", (const char*)line);
                    QString command = line.section(' ', 0, 0);
                    QString argument = line.section(' ', 1, 1);
                    if(command != "STORED") {
                        QMessageBox::information(this, "LOVE Command",
                                "Server does not reply with proper format!");
                    } else {
                        QString filename = QString(CLNTDIR "movie/love/%1")
                            .arg(argument);
                        m_movie->saveFile(filename);
                    }
                }
                ServerMotto->append(line);
            }
        }

        /* ------------------------------------------------
         * Response to the PLAY command
         * ------------------------------------------------ */
        if(m_state == StatePlay) {
            m_state = StateNone;
            emit signalConnected(true);
            if(line[0] == '-') {
                QMessageBox::information(this, "PLAY Command",
                        "Could not play this loveletter!");
            } else if(line[0] == '+') {
                QMessageBox::information(this, ":-)",
                        "Loveletter will play shortly!");
                continue;
            }
        }
    }
}

/*
 * public slot
 * ---The SERVER has closed connection
 */
void BlinkShareMain::slotConnectionClosed() {
    qDebug("Connection closed by foreign host");
    m_socket->close();
}

/*
 * public slot
 */
void BlinkShareMain::slotDelayedCloseFinished() {
    emit signalConnected(false);
    delete m_socket;
    m_socket = 0;
}


/*
 * public slot
 */
void BlinkShareMain::slotConnected() {
    qDebug("Connection up");
}

/*
 * Error in the current connection
 */
void BlinkShareMain::slotError(int) {
   
    delete m_socket;
    m_socket = 0;
    int ret = QMessageBox::warning(this, "Connection failed",
             "Re-Configure?",
             "Yes",
             "No",
            QString::null,
            1, 1);
    if(ret == 0) {
         slotConfigure();
    }
}

/*
 * The LIST-command
 */
void BlinkShareMain::slotList()
{
    emit signalConnected(false);
    m_state = StateList;
    QTextStream ts(m_socket);
    ts << "LIST\n";
    ServerListBox->clear();
    slotLocalList();
    slotLoveList();
}


/*
 * The local list
 */
void BlinkShareMain::slotLocalList()
{
    ClientListBox->clear();
    QDir dir(CLNTDIR "movie/share");
    if(!dir.exists()) {
        qFatal(CLNTDIR "movie/share DOES NOT EXIST!\n"
                "Bingo, you encountered a FATAL bug.\n"
                "Call Mr. Spock or your local package-manager!");
    }
    dir.setFilter(QDir::Files | QDir::NoSymLinks);

    const QFileInfoList* fileInfoList = dir.entryInfoList();
    if(!fileInfoList) {
        qFatal("Oh shit, fileInfoList not constructed. This is a bug. Bye!");
    }

    QFileInfoListIterator it(*fileInfoList);
    QFileInfo* fi;
    QString FileName;
    while((fi = it.current())) {
        FileName = fi->fileName().data();
        ClientListBox->insertItem(FileName);
        ++it;
    }
}

/*
 * The love list
 */
void BlinkShareMain::slotLoveList()
{
    LoveLetterBox->clear();
    QDir dir(CLNTDIR "movie/love");
    if(!dir.exists()) {
        qFatal(CLNTDIR "movie/love DOES NOT EXIST!\n"
                "Bingo, you encountered a FATAL bug.\n"
                "Call Mr. Spock or your local package-manager!");
    }
    dir.setFilter(QDir::Files | QDir::NoSymLinks);

    const QFileInfoList* fileInfoList = dir.entryInfoList();
    if(!fileInfoList) {
        qFatal("Oh shit, fileInfoList not constructed. This is a bug. Bye!");
    }

    QFileInfoListIterator it(*fileInfoList);
    QFileInfo* fi;
    QString FileName;
    while((fi = it.current())) {
        FileName = fi->fileName().data();
        LoveLetterBox->insertItem(FileName);
        ++it;
    }
}

/*
 * public slot
 */
void BlinkShareMain::slotUpOne()
{
    if(!c_send) {
        QMessageBox::information(this, "SEND Command",
                "SEND-command not supported by server!");
        return;
    }
    QString name = ClientListBox->text(ClientListBox->currentItem());
    QString filename = QString(CLNTDIR "movie/share/%1").arg(name);
    m_movie = new BlmMovie(name);
    if(!m_movie->loadFile(filename)) {
        QMessageBox::information(this, "Upload",
                "Could not read file!\n"
                "\n"
                "You should NEVER see this message...\n"
                "...it's a hidden feature!");
        return;
    }
    emit signalConnected(false);
    m_state = StateSend;
    QTextStream ts(m_socket);
    ts << "SEND " << name << "\n";
}

/*
 * public slot
 */
void BlinkShareMain::slotUpAll()
{
    if(!c_send) {
        QMessageBox::information(this, "SEND Command",
                "SEND-command not supported by server!");
        return;
    }
    emit signalConnected(false);
    QString* name;
    for(uint i=0; i < ClientListBox->count(); i++) {
       name = new QString(ClientListBox->text(i));
       m_queue.enqueue(name);
    }
    ServerMotto->clear();
    slotUpAllNext();
}

void BlinkShareMain::slotUpAllNext()
{
    if(m_queue.isEmpty()) {
        QString finished("** FINISHED UPLOAD **");
        ServerMotto->append(finished);
        m_state = StateNone;
        slotList();
        return;
    }
    QString* name;
    name = m_queue.dequeue();
    QString filename = QString(CLNTDIR "movie/share/%1").arg(*name);
    m_movie = new BlmMovie(*name);
    if(!m_movie->loadFile(filename)) {
        QString error = QString("Error reading file %1").arg(*name);
        delete m_movie;
        ServerMotto->append(error);
        slotUpAllNext();
        return;
    }
    m_state = StateSendAll;
    QTextStream ts(m_socket);
    ts << "SEND " << *name << "\n";
    delete name;
}

/*
 * public slot
 */
void BlinkShareMain::slotDownOne()
{
    if(!c_retr) {
        QMessageBox::information(this, "RETR Command",
                "RETR-command not supported by server!");
        return;
    }
    emit signalConnected(false);
    QString name = ServerListBox->text(ServerListBox->currentItem());
    m_movie = new BlmMovie(name);
    m_movie->startReadStream();
    m_state = StateRetr;
    QTextStream ts(m_socket);
    ts << "RETR " << name << "\n";
}

/*
 * public slot
 */
void BlinkShareMain::slotDownAll()
{
    if(!c_retr) {
        QMessageBox::information(this, "RETR Command",
                "RETR-command not supported by server!");
        return;
    }
    emit signalConnected(false);
    QString* name;
    for(uint i=0; i < ServerListBox->count(); i++) {
       name = new QString(ServerListBox->text(i));
       m_queue.enqueue(name);
    }
    ServerMotto->clear();
    slotDownAllNext();
}

/*
 * public slot
 */
void BlinkShareMain::slotDownAllNext()
{
    if(m_queue.isEmpty()) {
        QString finished("** FINISHED DOWNLOAD **");
        ServerMotto->append(finished);
        m_state = StateNone;
        slotList();
        return;
    }
    QString* name;
    name = m_queue.dequeue();
    QString filename = QString(CLNTDIR "movie/share/%1").arg(*name);
    m_movie = new BlmMovie(*name);
    m_movie->startReadStream();
    m_state = StateRetrAll;
    QTextStream ts(m_socket);
    ts << "RETR " << *name << "\n";
}

/*
 * The DELE command
 */
void BlinkShareMain::slotDelServer()
{
    if(!c_dele) {
        QMessageBox::information(this, "DELE Command",
                "DELE-command not supported by server!");
        return;
    }
    int ret = QMessageBox::warning(this, "Delete file",
             "Do you really want to delete this file?",
             "Yes",
             "No",
            QString::null,
            1, 1);
    if(ret != 0) {
        return;
    }
    emit signalConnected(false);
    QString name = ServerListBox->text(ServerListBox->currentItem());
    m_state = StateDele;
    QTextStream ts(m_socket);
    ts << "DELE " << name << "\n";
}

/*
 * public slot
 */
void BlinkShareMain::slotDelClient()
{
    int ret = QMessageBox::warning(this, "Delete file",
             "Do you really want to delete this file?",
             "Yes",
             "No",
            QString::null,
            1, 1);
    if(ret != 0) {
        return;
    }
    QString name = ClientListBox->text(ClientListBox->currentItem());
    QString filename = QString(CLNTDIR "movie/share/%1").arg(name);
    QFile file(filename);
    if(!file.remove()) {
        QMessageBox::information(this, "Delete",
                "Could not delete file!\n"
                "\n"
                "You should NEVER see this message...\n"
                "...it's a hidden feature!");
        return;
    }
    slotLocalList();
}

/*
 * public slot
 */
void BlinkShareMain::slotDelLove()
{
    int ret = QMessageBox::warning(this, "Delete LOVELETTER",
             "Do you really want to delete this LOVELETTER?",
             "Yes",
             "No",
            QString::null,
            1, 1);
    if(ret != 0) {
        return;
    }
    QString name = LoveLetterBox->text(LoveLetterBox->currentItem());
    QString filename = QString(CLNTDIR "movie/love/%1").arg(name);
    QFile file(filename);
    if(!file.remove()) {
        QMessageBox::information(this, "Delete",
                "Could not delete file!\n"
                "\n"
                "You should NEVER see this message...\n"
                "...it's a hidden feature!");
        return;
    }
    slotLoveList();
}

/*
 * The INFO command
 */
void BlinkShareMain::slotInfoServer()
{
    if(!c_info) {
        QMessageBox::information(this, "INFO Command",
                "INFO-command not supported by server!");
        return;
    }
    emit signalConnected(false);
    QString name = ServerListBox->text(ServerListBox->currentItem());
    m_state = StateInfo;
    QTextStream ts(m_socket);
    ts << "INFO " << name << "\n";
    ServerMotto->clear();
}

/*
 * public slot
 */
void BlinkShareMain::slotInfoClient()
{
    QString name = ClientListBox->text(ClientListBox->currentItem());
    QString filename = QString(CLNTDIR "movie/share/%1").arg(name);
    QFile file(filename);
    if(!file.open(IO_ReadOnly)) {
        QMessageBox::information(this, "Info",
                "Could not read file!\n"
                "\n"
                "You should NEVER see this message...\n"
                "...it's a hidden feature!");
        return;
    }
    QTextStream filestream(&file);
    QString line;
    ServerMotto->clear();
    while(!filestream.eof()) {
        line = filestream.readLine();
        if(line[0] != '#') {
            break;
        }
        ServerMotto->append(line);
    }
    file.close();
}

/*
 * public slot
 */
void BlinkShareMain::slotInfoLove()
{
    QString name = LoveLetterBox->text(LoveLetterBox->currentItem());
    QString filename = QString(CLNTDIR "movie/love/%1").arg(name);
    QFile file(filename);
    if(!file.open(IO_ReadOnly)) {
        QMessageBox::information(this, "Info",
                "Could not read file!\n"
                "\n"
                "You should NEVER see this message...\n"
                "...it's a hidden feature!");
        return;
    }
    QTextStream filestream(&file);
    QString line;
    ServerMotto->clear();
    while(!filestream.eof()) {
        line = filestream.readLine();
        if(line[0] != '#') {
            break;
        }
        ServerMotto->append(line);
    }
    file.close();
}

/*
 * public slot
 */
void BlinkShareMain::slotPlayLove()
{
    QString name = LoveLetterBox->text(LoveLetterBox->currentItem());
    QTextStream ts(m_socket);
    emit signalConnected(false);
    m_state = StatePlay;
    ts << "PLAY " << name << "\n";
}

/*
 * public slot
 */
void BlinkShareMain::slotUpLove()
{
    if(!c_love) {
        QMessageBox::information(this, "LOVE Command",
                "LOVE-command not supported by server!");
        return;
    }
    QString name = ClientListBox->text(ClientListBox->currentItem());
    QString filename = QString(CLNTDIR "movie/share/%1").arg(name);
    m_movie = new BlmMovie(name);
    if(!m_movie->loadFile(filename)) {
        QMessageBox::information(this, "Upload LOVE",
                "Could not read file!\n"
                "\n"
                "You should NEVER see this message...\n"
                "...it's a hidden feature!");
        return;
    }
    emit signalConnected(false);
    m_state = StateLove;
    QTextStream ts(m_socket);
    ts << "LOVE\n";
}


