#ifndef BLMFILE_H
#define BLMFILE_H

#include <qstring.h>
#include <qdatetime.h>
#include <qobject.h>
#include <qintdict.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qdatastream.h>
#include <qbuffer.h>

#ifdef QT_THREAD_SUPPORT
#include <qthread.h>
#endif //QT_THREAD_SUPPORT

#define BLMHEIGHT 8
#define BLMWIDTH 18

class BlmFrame {

public:

    bool pixel[BLMWIDTH][BLMHEIGHT];
    long delay;

    void clear() {
        delay = -1;
        int height, width;
        for(height = 0; height < BLMHEIGHT; height++) {
            for(width = 0; width < BLMWIDTH; width++) {
                pixel[width][height] = false;
            }
        }
    }

    void debug() {
        QString debstr;
        int height, width;
        for(height = 0; height < BLMHEIGHT; height++) {
            for(width = 0; width < BLMWIDTH; width++) {
                if(pixel[width][height] == false) {
                    debstr.append(".");
                } else {
                    debstr.append("#");
                }
            }
            debstr.append("\n");
        }
        debstr.append("------------------\n");
        qDebug("Delay: %ld", delay);
        qDebug("%s", (const char *)debstr);
    }


};

class BlmMovie {

public:
    BlmMovie(QString& name) {
        m_name = name;
        m_frames.setAutoDelete(true);
        m_comments.setAutoDelete(true);
        m_framenum = 0;
    }

    void writeTextStream(QTextStream& filestream) {
        QBuffer buffer(m_array);
        buffer.open(IO_ReadOnly);
        QTextStream arraystream(&buffer);
        QString line;
#ifdef QT_THREAD_SUPPORT
        int loopcount = 0;
#endif //QT_THREAD_SUPPORT

        while(!arraystream.eof()) {
#ifdef QT_THREAD_SUPPORT
            // If we are using Threading, we fake a
            // much, much lower priority for this thread
            // so the GUI-Thread can handle timing a bit better
            loopcount++;
            if(loopcount == 10) {
                //QThread::msleep(30);
                loopcount = 0;
            }
#endif //QT_THREAD_SUPPORT
            line = arraystream.readLine();
            filestream << line << "\n";
        }
    }

    void writeInfoStream(QTextStream& filestream) {
        QBuffer buffer(m_array);
        buffer.open(IO_ReadOnly);
        QTextStream arraystream(&buffer);
        QString line;

        while(!arraystream.eof()) {
            line = arraystream.readLine();
            if(line[0] != '#') {
                break;
            }
            filestream << line << "\n";
        }
    }

    void startReadStream() {
        m_array.resize(0);
        m_buffer = new QBuffer(m_array);
        m_buffer->open(IO_WriteOnly);
        m_stream = new QTextStream(m_buffer);
    }

    void addLineReadStream(QString& line) {
        *m_stream << line << "\n";
    }

    void stopReadStream() {
        m_buffer->close();
        delete m_buffer;
        delete m_stream;
    }
        
    bool loadFile(QString& filename) {
        m_array.resize(0);
        QFile file(filename);
        if(!file.open(IO_ReadOnly)) {
            return false;
        }
        QTextStream filestream(&file);
        QBuffer buffer(m_array);
        buffer.open(IO_WriteOnly);
        QTextStream arraystream(&buffer);
        QString line;

        while(!filestream.eof()) {
            line = filestream.readLine();
            arraystream << line << "\n";
        }
        
        buffer.close();
        file.close();
        
        return parseFile();
    }

    bool loadFromArray(QByteArray& array) {
        m_array = array;
        
        return parseFile();
    }

    QByteArray saveToArray() {
        return m_array;
    }
        
    bool saveFile(QString& filename) {
        QFile file(filename);
        if(!file.open(IO_WriteOnly)) {
            return false;
        }
        QTextStream filestream(&file);
        QBuffer buffer(m_array);
        buffer.open(IO_ReadOnly);
        QTextStream arraystream(&buffer);
        QString line;

        while(!arraystream.eof()) {
            line = arraystream.readLine();
            filestream << line << "\n";
        }
        
        buffer.close();
        file.close();
        return true;
    }
        
    bool parseFile() {
        QBuffer buffer(m_array);
        buffer.open(IO_ReadOnly);
        QTextStream stream(&buffer);
        QString line;
        BlmFrame* myframe = new BlmFrame();
        int FrameLines = 0;
        int counter;
        QString stripped;
        while(!stream.eof()) {
            line = stream.readLine();
            if(line.startsWith("#")) {
                m_comments.insert((long)m_comments.count(), 
                        new QString(line));
                continue;
            }
            if(line.startsWith("@")) {
                if(FrameLines == 0) {
                    stripped = "";
                    int loop;
                    for(loop = 1; loop < (long)line.length(); loop++) {
                        stripped.append(line[loop]);
                    }
                    myframe->delay = stripped.toLong();
                    continue;
                } else {
                    qDebug("Unexpectet TIME-Line (@)");
                    buffer.close();
                    return false;
                }
            } else {
                if(FrameLines > BLMHEIGHT) {
                    qDebug("Error - too many # of lines in Frame");
                    buffer.close();
                    return false;
                }
            }

            if(line.isEmpty()) {
                if(FrameLines == 0) {
                    continue;
                } else if(FrameLines == BLMHEIGHT) {
                    addFrame(myframe);
                    myframe = new BlmFrame();
                    FrameLines = 0;
                    continue;
                } else {
                    qDebug("Error - wrong # of lines in Frame");
                    buffer.close();
                    return false;
                }
            }

            if(line.length() != BLMWIDTH) {
                qDebug("--> %s", (const char*)line);
                qDebug("Error - wrong length of line IS: %d SHOULD: %d",
                        line.length(), BLMWIDTH);
                buffer.close();
                return false;
            }

            for(counter = 0; counter < (int)line.length(); counter++) {
                if(line[counter] == '1') {
                    myframe->pixel[counter][FrameLines] = true; 
                } else {
                    myframe->pixel[counter][FrameLines] = false; 
                }
            }
            FrameLines++;
            
        }

        buffer.close();
        //
        return true;
    }

    void setFirst(void) {
        m_framenum = 0;
    }

    BlmFrame* getFrame(void) {
        m_framenum++;
        //if(m_framenum > 5) return 0;
        if(m_framenum >= (long)m_frames.count()) {
            return 0;
        } else {
            return m_frames.find(m_framenum);
        }
    }

    void addFrame(BlmFrame* frame) {
        m_frames.insert((long)m_frames.count(), frame);
    }

    QString getName() {
        return m_name;
    }
        

private:
   QIntDict<BlmFrame> m_frames;
   QIntDict<QString> m_comments;
   long m_framenum;
   QString m_name;
   QByteArray m_array;
   QBuffer* m_buffer;
   QTextStream* m_stream;
}; 

#endif // BLMFILE_H
