#include <qlabel.h>
#include <qevent.h>

#include "consoleconnection.h"
#include "global.h"
#include "messenger.h"

/* 
 *  Constructs a ConsoleConnection which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 */
ConsoleConnection::ConsoleConnection( bool notify, bool show, 
        QWidget* parent,  const char* name, WFlags fl )
    : consoleconnectionbase( parent, name, fl )
{
    m_count = 0;
    m_notify = notify;
    m_show = show;
    connect(g_messenger, SIGNAL(signalBlmFrame(BlmFrame&)),
            this, SLOT(slotShowFrame(BlmFrame&)));

    g_messenger->slotLogEntry(LogEntry::StatusINFO, 
            "Starting new Console-Connection");

    m_closeignore = true;
}

/*  
 *  Destroys the object and frees any allocated resources
 */
ConsoleConnection::~ConsoleConnection()
{
    // no need to delete child widgets, Qt does it all for us
}

/*
 * public slot
 */
void ConsoleConnection::slotShowFrame(BlmFrame& frame)
{
    m_count++;
    QString count = QString("Frames: %1").arg( m_count, 8, 10 );
    FrameCountLabel->setText(count);
    if(m_notify) {
        qDebug("Got Frame %ld", m_count);
    }
    if(m_show) {
        frame.debug();
    }
}

/*
 * public slot
 */
void ConsoleConnection::slotClose()
{
    g_messenger->slotLogEntry(LogEntry::StatusINFO, "Closing Console-Connection");
    m_closeignore = false;
    close();
}

void ConsoleConnection::closeEvent(QCloseEvent* event)
{
    if(m_closeignore) {
        this->slotClose();
        if(m_closeignore) {
            event->ignore();
        } else {
            event->accept();
        }
    } else {
        event->accept();
    }
}
