#include <qlabel.h>
#include <qevent.h>

#include "dummyconnection.h"
#include "global.h"
#include "messenger.h"

/* 
 *  Constructs a DummyConnection which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 */
DummyConnection::DummyConnection( QWidget* parent,  const char* name, WFlags fl )
    : dummyconnectionbase( parent, name, fl )
{
    m_count = 0;
    m_closeignore = true;
    connect(g_messenger, SIGNAL(signalBlmFrame(BlmFrame&)),
            this, SLOT(slotBlmFrame()));

    g_messenger->slotLogEntry(LogEntry::StatusINFO, "Starting new Dummy-Connection");
}

/*  
 *  Destroys the object and frees any allocated resources
 */
DummyConnection::~DummyConnection()
{
    // no need to delete child widgets, Qt does it all for us
}

/*
 * public slot
 */
void DummyConnection::slotClose()
{
    g_messenger->slotLogEntry(LogEntry::StatusINFO, "Closing Dummy-Connection");
    m_closeignore = false;
    close();
}

void DummyConnection::closeEvent(QCloseEvent* event)
{
    if(m_closeignore) {
        this->slotClose();
        if(m_closeignore) {
            event->ignore();
        } else {
            event->accept();
        }
    } else {
        event->accept();
    }
}


void DummyConnection::slotBlmFrame() {
    m_count++;
    QString count = QString("Frames: %1").arg( m_count, 8, 10 );
    FrameCountLabel->setText(count);
}
