#include <qlistbox.h>
#include <qdir.h>
#include <qfileinfo.h>
#include "global.h"
#include "playlist.h"
#include "playlistentry.h"
#include "blmfile.h"
#include "messenger.h"

#include <stdlib.h>
#include <time.h>
#include <qtimer.h>

/*
 * public slot
 */

Playlist::Playlist(QListBox* _parent, long defaultfill) {
    m_parent = _parent;
    m_defaultfill = defaultfill;
    m_PlayList.setAutoDelete(true);
    m_lowest = 1;
    m_highest = 0;
    m_lovelowest = 1;
    m_lovehighest = 0;
    m_current = 0;
    m_skip = false;
    this->fillList(0);
    srand(time(0));

    connect(this, SIGNAL(signalShowFrame(BlmFrame&)), 
            g_messenger, SLOT(slotBlmFrame(BlmFrame&)));

    m_timer = new QTimer(this);
    connect(m_timer, SIGNAL(timeout()), this, SLOT(slotTimeOut()));
    m_timer->start(3000, true);

}

void Playlist::skip() {
    m_skip = true;
}

void Playlist::slotTimeOut(void) {
    if(m_movie == 0) {
        QString nextMovie = getNext();
        if(nextMovie == 0) {
            m_timer->start(2000, true);
            return;
        }
        m_movie = new BlmMovie(nextMovie);
        if(!m_movie->loadFile(nextMovie)) {
            delete m_movie;
            m_movie=0;
            m_timer->start(0, true);
            return;
        }
    }
    
    if(m_skip) {
        m_skip = false;
        delete m_movie;
        m_movie=0;
        m_timer->start(0, true);
        return;
    }
       
    BlmFrame* frame = m_movie->getFrame();
    if(frame == 0) {
        delete m_movie;
        m_movie=0;
        m_timer->start(0, true);
        return;
    }
        
    /* We start the timer FIRST, so we get more accurate
     * timing (emiting the signal calls MANY functions on a loaded system) */
    m_timer->start(frame->delay, true);
    emit signalShowFrame(*frame);

}

void Playlist::fillList(long fill=0) {
    if(fill < m_defaultfill) {
        fill = m_defaultfill;
    }

    if(fill < (long)m_PlayList.count()) {
        return;
    }

    QDir dir(BASEDIR "movie/share");
    dir.setFilter(QDir::Files | QDir::NoSymLinks);

    const QFileInfoList* fileInfoList = dir.entryInfoList();
    QFileInfoListIterator it(*fileInfoList);
    QFileInfo* fi;

    QString FilePath;
    QString FileName;
    QIntDict<PlaylistEntry> Playlist;
    Playlist.setAutoDelete(true);
    PlaylistEntry *entry;
    long count=1;

    while((fi = it.current())) {
        QString FilePath = fi->absFilePath().data();
        QString FileName = fi->fileName().data();
        Playlist.insert(count, new PlaylistEntry(FilePath,
                    FileName, false, true));
        count++;

        //this->add(new PlaylistEntry(FilePath,
        //            FileName, false, true));
        ++it;
    }

    long myrand;
    while((long)m_PlayList.count() < fill) {
        myrand = (long)(((double)rand()/ RAND_MAX)*count);
        entry = Playlist.take(myrand);
        if(entry != 0) {
            this->add(entry);
        }
    }

    //delete fileInfoList;
}

QString Playlist::getNext() {
    if(m_current != 0) {
        delete m_current;
        m_current = 0;
    }
    if(m_PlayList.count() < 2) {
        this->fillList(0);
    }
        
    if(m_lovelowest <= m_lovehighest) {
        m_current = m_LovePlayList.take(m_lovelowest);
        m_lovelowest++;
        this->update();
        return(m_current->file);
    }
    if(m_lowest <= m_highest) {
        m_current = m_PlayList.take(m_lowest);
        m_lowest++;
        this->update();
        return(m_current->file);
    }
        

    qDebug("???Something fucked up: LO: %ld HI: %ld",
                    m_lowest, m_highest);
    return 0;
}

void Playlist::update() {
    m_parent->clear();
    long counter;
    PlaylistEntry* entry;
    QString name;

    if(m_current != 0) {
        name = "#  ";
        name.append(m_current->name);
        new QListBoxText(m_parent, name);
    }

    for(counter = m_lovelowest; counter <= m_lovehighest; counter++) {
        entry = m_LovePlayList.find(counter);
        name = " L ";
        name.append(entry->name);
        new QListBoxText(m_parent, name);
    }
    
    for(counter = m_lowest; counter <= m_highest; counter++) {
        entry = m_PlayList.find(counter);
        if(entry == 0) {
            qDebug("Something fucked up: LO: %ld HI: %ld CO: %ld",
                    m_lowest, m_highest, counter);
        }
        if(entry->isAuto) {
            name = "   ";
        } else {
            name = " ->";
        }
        name.append(entry->name);
        new QListBoxText(m_parent, name);
    }
}

void Playlist::add(PlaylistEntry* entry) {
    if(entry->isLoveletter == true) {
        m_lovehighest++;
        m_LovePlayList.insert(m_lovehighest, entry);
    } else {
        m_highest++;
        m_PlayList.insert(m_highest, entry);
    }
}

