#include <qlabel.h>
#include <qsocket.h>
#include <qtextstream.h>
#include <qtimer.h>
#include <qstring.h>
#include <qdir.h>
#include <qfileinfo.h>
#include <stdlib.h>

#include "sbtpchildsocket.h"
#include "global.h"
#include "messenger.h"
#include "playlist.h"
#include "playlistentry.h"
#include "blmfile.h"

/*
 * The SbtpChildSocket class provides a socket that is connected with a client.
 * For every client that connects to the server, the server creates a new
 * instance of this class.
 */


SbtpChildSocket::SbtpChildSocket( int sock, bool full,
        SbtpChildWidget *parent=0, const char *name=0 ) :
            QSocket( parent, name )
{
    connect( this, SIGNAL(connectionClosed()),
        SLOT(deleteLater()) );
    connect( this, SIGNAL(connectionClosed()),
        this, SLOT(slotEmitClose()) );
    connect( this, SIGNAL(error(int)),
        this, SLOT(slotEmitClose()) );
    connect( this, SIGNAL(signalClosing()),
        parent, SLOT(slotRemoveChild()) );
    connect( this, SIGNAL(signalDestruction()),
        parent, SLOT(slotDestruction()) );


    setSocket( sock );

    if(full) {
        QTimer* timer = new QTimer(this);
        connect(timer, SIGNAL(timeout()), this, SLOT(closeClient()));
        timer->start(100, true);
    } else {
        // Initializing!
        m_stagehelo = true;
        m_helogiven = false;
        m_sendfile = false;
        m_loveletter = false;
        m_user = "";
        m_pass = "";
        QTextStream ts( this );
        ts << "+BlinkServ CONDOR ready, i speak Sbtp 1.0\n";

        ts << "                          ::::::::  :::::::::: :::::::::  :::     :::\n";
        ts << "                        :+:    :+: :+:        :+:    :+: :+:     :+:\n";
        ts << "__ )  |_)       |      +:+        +:+        +:+    +:+ +:+     +:+\n";
        ts << "__ \\  | | __ \\  |  /  +#++:++#++ +#++:++#   +#++:++#:  +#+     +:+\n";
        ts << "|   | | | |   |   <         +#+ +#+        +#+    +#+  +#+   +#+\n";
        ts << "___/ _|_|_|  _|_|\\_\\#+#    #+# #+#        #+#    #+#   #+#+#+#\n";
        ts << "                    ########  ########## ###    ###     ###\n";
        ts << "\n";
        ts << "         .....144 pixel should be enough for everyone.....\n";
        ts << ".\n";

        connect( this, SIGNAL(readyRead()),
            SLOT(doWork()) );
    }


}

SbtpChildSocket::~SbtpChildSocket()
{
    emit signalDestruction();
}


void SbtpChildSocket::doWork()
{
    QTextStream ts( this );
    QString line;
    QString command;
    QString param;
    while(canReadLine()) {
        line = ts.readLine();

        // Multi-Line responses from client
        if(m_sendfile || m_loveletter) {
            if(line == ".") {
                m_movie->stopReadStream();
                if(!m_movie->parseFile()) {
                    delete m_movie;
                    ts << "-Sorry, file does not parse\n";
                    flush();
                    m_sendfile = false;
                    m_loveletter = false;
                    continue;
                }
                if(!m_movie->saveFile(m_filename)) {
                    ts << "-Could not save file\n";
                } else {
                    ts << "+Gotcha - file succesfully saved\n";
                }
                if(m_loveletter) {
                    ts << "STORED " << m_shortfilename << "\n";
                    ts << "You can use the PLAY-command to\n";
                    ts << "start this loveletter\n";
                    ts << "\n";
                    ts << "Please rememember, you need the\n";
                    ts << "ID " << m_shortfilename << " to play\n";
                    ts << "the loveletter!\n";
                    ts << ".\n";
                } else {
                    g_playlist->add(new PlaylistEntry(m_filename,
                                        m_shortfilename, false, false));
                    g_playlist->update();

                }
                flush();
                m_sendfile = false;
                m_loveletter = false;
                delete m_movie;
                continue;
            } else {
                m_movie->addLineReadStream(line);
                continue;
            }
        }

        // Standard commands
        if(!parseLine(line, command, param)) {
            qDebug("---Parse error: ->%s<-", (const char*)line);
            ts << "-Could not parse command\n";
            continue;
        }

        // QUIT works ALWAYS!!!!!
        if(command == "QUIT") {
            ts << "+Bye\n";
            flush();
            emit signalClosing();
            close();
            break;
        }

        if(m_stagehelo) {
        /* ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
         *          Authentification stage
         * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
            // HELO-Command - can only be given *ONCE*!
            if(command == "HELO") {
                if(m_helogiven) {
                    ts << "-HELO can not be issued TWICE, signing (you) off\n";
                    flush();
                    emit signalClosing();
                    close();
                    break;
                } else {
                    m_helogiven = true;
                    ts << "+These are the commands i understand\n";
                    ts << "COMD USER\n";
                    ts << "COMD PASS\n";
                    ts << "COMD QUIT\n";
                    ts << "COMD LIST\n";
                    ts << "COMD RETR\n";
                    ts << "COMD INFO\n";
                    ts << "COMD SEND\n";
                    ts << "COMD LOVE\n";
                    ts << "COMD PLAY\n";
                    ts << "COMD DELE\n";
                    ts << "FILE BLM\n";
                    ts << "LGIN ANON\n";
                    ts << ".\n";
                    flush();
                    continue;
                }
            }
            if(!m_helogiven) {
                ts << "-No. Wont do that. Not know. Not until you say HELO!\n";
                flush();
                continue;
            }

            // USER-command... you'll never guess what that does!
            if(command == "USER") {
                if(m_user.length() > 0) {
                    ts << "-User already given, ignoring command\n";
                    flush();
                    continue;
                }
                if(param.length() == 0) {
                    ts << "-Need username after USER!\n";
                    flush();
                    continue;
                }
                ts << "+Stored username, now gimme a PASS\n";
                m_user = param;
                flush();
                continue;
            }

            // Security by obsurity? No, none security at all
            if(command == "PASS") {
                if(m_user.length() == 0) {
                    ts << "-No USER given, ignoring command\n";
                    flush();
                    continue;
                }
                if(param.length() == 0) {
                    ts << "-Need password after PASS!\n";
                    flush();
                    continue;
                }
                m_pass = param;
                if((m_user == "anonymous") && (m_pass == "anonymous")) {
                    ts << "+Welcome to file-transfer mode, user anonymous\n";
                    m_stagehelo = false;
                    continue;
                } else {
                    ts << "-You didn't say the magic word! Bye, hacker! :-)\n";
                    emit signalClosing();
                    close();
                    flush();
                    break;
                }
            }
        } else {
        /* ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
         *          Authentification stage
         * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
            if(command == "LIST") {
                ts << "+I have these file for you:\n";
                QDir dir(BASEDIR "movie/share");
                dir.setFilter(QDir::Files | QDir::NoSymLinks);

                const QFileInfoList* fileInfoList = dir.entryInfoList();
                QFileInfoListIterator it(*fileInfoList);
                QFileInfo* fi;

                QString FileName;
                while((fi = it.current())) {
                    FileName = fi->fileName().data();
                    ts << FileName << "\n";
                    ++it;
                }
                ts << ".\n";
                flush();
                continue;


            }
            if(command == "RETR") {
               if(param.length() == 0) {
                   ts << "-don't you think a filename would help a bit?\n";
                   flush();
                   continue;
               }
               if(param.length() < 5) {
                   ts << "-need the full filename\n";
                   flush();
                   continue;
               }
               if(!checkFilename(param)) {
                   ts << "-Illegal characters in filename\n";
                   flush();
                   continue;
               }
               m_movie = new BlmMovie(param); 
               QString filename = QString(BASEDIR "movie/share/%1").arg(param);
               if(!m_movie->loadFile(filename)) {
                   ts << "-could not open file\n";
                   flush();
                   continue;
               } else {
                   ts << "+Here you are\n";
                   m_movie->writeTextStream(ts);
                   ts << ".\n";
                   flush();
                   continue;
               }
               delete m_movie;
            }
            if(command == "INFO") {
               if(param.length() == 0) {
                   ts << "-don't you think a filename would help a bit?\n";
                   flush();
                   continue;
               }
               if(param.length() < 5) {
                   ts << "-need the full filename\n";
                   flush();
                   continue;
               }
               if(!checkFilename(param)) {
                   ts << "-Illegal characters in filename\n";
                   flush();
                   continue;
               }
               m_movie = new BlmMovie(param); 
               QString filename = QString(BASEDIR "movie/share/%1").arg(param);
               if(!m_movie->loadFile(filename)) {
                   ts << "-could not open file\n";
                   flush();
                   continue;
               } else {
                   ts << "+File information avaiable\n";
                   m_movie->writeInfoStream(ts);
                   ts << ".\n";
                   flush();
                   continue;
               }
               delete m_movie;
            }
            if(command == "DELE") {
               if(param.length() == 0) {
                   ts << "-don't you think a filename would help a bit?\n";
                   flush();
                   continue;
               }
               if(param.length() < 5) {
                   ts << "-need the full filename\n";
                   flush();
                   continue;
               }
               if(!checkFilename(param)) {
                   ts << "-Illegal characters in filename\n";
                   flush();
                   continue;
               }
               QString filename = QString(BASEDIR "movie/share/%1").arg(param);
               QFile file(filename);
               if(file.remove()) {
                   ts << "+file removed\n";
               } else {
                   ts << "-could not remove file\n";
               }
               flush();
               continue;
            }
            if(command == "SEND") {
               if(param.length() == 0) {
                   ts << "-don't you think a filename would help a bit?\n";
                   flush();
                   continue;
               }
               if(param.length() < 5) {
                   ts << "-need the full filename\n";
                   flush();
                   continue;
               }
               if(!checkFilename(param)) {
                   ts << "-Illegal characters in filename\n";
                   flush();
                   continue;
               }
               m_movie = new BlmMovie(param); 
               m_filename = QString(BASEDIR "movie/share/%1").arg(param);
               m_shortfilename = param;
               m_movie->startReadStream();
               ts << "+Gimme file\n";
               flush();
               m_sendfile = true;
               continue;
            }
            if(command == "LOVE") {
               if(param.length() != 0) {
                   ts << "-Do NOT send filenames for LOVE\n";
                   flush();
                   continue;
               }
               long myrand;
               
               // Search for a file that does NOT exist
               while(true) {
                   myrand = (long)(((double)rand()/ RAND_MAX)*100000)+100000;
                   m_shortfilename = QString("%1").arg(myrand);
                   m_filename = QString(BASEDIR "movie/love/%1").arg(m_shortfilename);
                   if(!QFile::exists(m_filename)) {
                       break;
                   }
               }
                   
               m_movie = new BlmMovie(m_shortfilename); 
               m_movie->startReadStream();
               ts << "+Loveletter subsystem operational\n";
               flush();
               m_loveletter = true;
               continue;
            }
            if(command == "PLAY") {
               if(param.length() == 0) {
                   ts << "-don't you think an ID would help a bit?\n";
                   flush();
                   continue;
               }
               if(!checkFilename(param)) {
                   ts << "-Illegal characters in filename\n";
                   flush();
                   continue;
               }

               m_shortfilename = QString("%1").arg(param);
               m_filename = QString(BASEDIR "movie/love/%1").arg(m_shortfilename);
               qDebug("Trying filename %s", (const char *)m_filename);
               if(!QFile::exists(m_filename)) {
                   ts << "-Sorry, i don't have such a loveletter\n";
               } else {
                    g_playlist->add(new PlaylistEntry(m_filename,
                                        m_shortfilename, true, false));
                    g_playlist->update();
                   ts << "+Loveletter added to Priority-Playlist\n";
               }
               flush();
               continue;
            }
        }
                    
    }
}

bool SbtpChildSocket::checkFilename(QString& filename) {
    if((filename.contains("/")) ||
           (filename.contains("%")) ||
           (filename.contains("\\")) ||
           (filename.contains("~")) ||
           (filename.contains(".."))) {
        return false;
    } else {
        return true;
    }
}

bool SbtpChildSocket::parseLine(QString& line,
                                QString& command,
                                QString& param) {
    if((line.length() < 4) || (line.length() > 120) || (line.length() == 5)) {
        return false;
    }
    if(line.length() == 4) {
        command = line.upper();
        param = "";
        return true;
    }

    QString dummy;
    dummy = line.section(' ', 0, 0);
    if(dummy.length() != 4) {
        return false;
    }
    command = dummy.upper();
    param = line.section(' ', 1, 1);
    return true;
}
