#ifndef SBTPCHILDSOCKET_H
#define SBTPCHILDSOCKET_H
#include <qobject.h>
#include <qlabel.h>
#include <qsocket.h>
#include <qtextstream.h>
#include <qtimer.h>
#include <qstring.h>

#include "global.h"
#include "messenger.h"
#include "blmfile.h"
#include "sbtpchildwidget.h"

/*
 * The SbtpChildSocket class provides a socket that is connected with a client.
 * For every client that connects to the server, the server creates a new
 * instance of this class.
 */
class SbtpChildSocket : public QSocket
{
    Q_OBJECT

public:
    SbtpChildSocket( int sock, bool full,
            SbtpChildWidget *parent=0, const char *name=0 );

    ~SbtpChildSocket();

public slots:

    void slotEmitClose() {
        emit signalClosing();
    }

private:
    void showFull() {
        QTextStream ts( this );
        ts << "Sorry, server is full, try again later!\n";
        emit signalClosing();
        this->close();
    }
    bool parseLine(QString&, QString&, QString&);
    bool checkFilename(QString&);


signals:
    void signalClosing();
    void signalDestruction();


private slots:
    void doWork();

private:
    bool m_stagehelo;
    bool m_helogiven;
    bool m_loveletter;
    bool m_sendfile;
    QString m_filename;
    QString m_shortfilename;
    QString m_user;
    QString m_pass;
    BlmMovie* m_movie;
};

#ifdef QT_THREAD_SUPPORT

#include <qthread.h>

/*
 * The SbtpChildSocket class provides a socket that is connected with a client.
 * For every client that connects to the server, the server creates a new
 * instance of this class.
 */
class SbtpChildThread : public QThread
{

public:
    SbtpChildThread( int sock, bool full,
            SbtpChildWidget *parent=0, const char *name=0 ) {

        m_sock = sock;
        m_full = full;
        m_parent = parent;
    }

    void run() {
        m_child = new SbtpChildSocket(m_sock, m_full, m_parent);
    }

    ~SbtpChildThread() {};

private:
    int m_sock;
    bool m_full;
    SbtpChildWidget* m_parent;
    SbtpChildSocket* m_child;

};
#endif // QT_THREAD_SUPPORT

#endif //SBTPCHILDSOCKET_H
