#include <qlabel.h>
#include "sbtpchildwidget.h"
#include "sbtpmotherwidget.h"
#include "sbtpchildsocket.h"

#include "global.h"
#include "messenger.h"
#include "blmfile.h"

/* 
 *  Constructs a SbtpChildWidget which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 */
SbtpChildWidget::SbtpChildWidget(  int socket, bool full,
        SbtpMotherWidget* serverparent,
        QWidget* parent,  const char* name, WFlags fl )
    : SbtpChildWidgetBase( parent, name, fl )
{
    PortLabel->setText(QString("Port: %1").arg(serverparent->getPort()));
    connect(serverparent, SIGNAL(signalCloseAll()),
            this, SLOT(slotClose()));
#ifdef QT_THREAD_SUPPORT
    qDebug("Threading SbtpChildSocket");
    m_child = new SbtpChildThread(socket, full, this);
#else // QT_THREAD_SUPPORT
    qDebug("Non-threading SbtpChildSocket");
    m_child = new SbtpChildSocket(socket, full, this);
#endif // QT_THREAD_SUPPORT
    /*
    connect( m_child, SIGNAL(signalClosing()),
                this, SLOT(slotRemoveChild()) );
    connect( m_child, SIGNAL(signalDestruction()),
                this, SLOT(slotDestruction()) );
    */  
    connect(this, SIGNAL(signalRemoveChild()),
            serverparent, SLOT(slotRemoveChild()));

#ifdef QT_THREAD_SUPPORT
#include <qthread.h>
    m_child->start();
#endif // QT_THREAD_SUPPORT

    m_count = 0;
    m_closesent = false;

}

/*  
 *  Destroys the object and frees any allocated resources
 */
SbtpChildWidget::~SbtpChildWidget()
{
    if(m_child) {
        delete m_child;
    }
}

/*
 * public slot
 */
void SbtpChildWidget::slotClose()
{
    if(!m_closesent) {
        m_closesent = true;
        emit signalRemoveChild();
    }
    close();
}

/*
 * public slot
 */
void SbtpChildWidget::slotRemoveChild()
{
    slotClose();
}

void SbtpChildWidget::slotDestruction()
{
    m_child = 0;
    slotRemoveChild();
}


