#include <qevent.h>
#include <qspinbox.h>
#include <qcheckbox.h>
#include <qlabel.h>
#include <qworkspace.h>

#include "sbtpmotherwidget.h"
#include "global.h"
#include "messenger.h"
#include "sbtpmothersocket.h"

/* 
 *  Constructs a SbtpMotherWidget which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 */
SbtpMotherWidget::SbtpMotherWidget(  int port, int maxconnections, 
        QWidget* parent,  const char* name, WFlags fl )
    : SbtpMotherWidgetBase( parent, name, fl )
{

    m_port = port;
    QString porttext = QString("Port: %1").arg( m_port, 5, 10 );
    PortLabel->setText(porttext);

    m_maxconn = maxconnections;
    MaxConnectionsBox->setValue(m_maxconn);

    g_messenger->slotLogEntry(LogEntry::StatusINFO,
            QString("Starting Sbtp-Server Port %1, max. Connections %2")
            .arg(m_port).arg(m_maxconn));

    g_messenger->slotLogEntry(LogEntry::StatusINFO, "Sbtp-Server is running");

    m_socket = new SbtpMotherSocket(m_port, this);
    connect(m_socket, SIGNAL(signalAddChild(int)), 
            this, SLOT(slotAddChild(int)));
    connect(m_socket, SIGNAL(signalRemoveChild()), 
            this, SLOT(slotRemoveChild()));

    m_closeignore = true;
    m_count = 0;
    m_childs = 0;

}

/*  
 *  Destroys the object and frees any allocated resources
 */
SbtpMotherWidget::~SbtpMotherWidget()
{
    // no need to delete child widgets, Qt does it all for us
}

/*
 * public slot
 */
void SbtpMotherWidget::slotClose()
{
    g_messenger->slotLogEntry(LogEntry::StatusINFO,
            QString("Closing Sbtp-Server Port %1").arg(m_port));
    emit signalCloseAll();
    delete m_socket;
    m_closeignore = false;
    close();
}

void SbtpMotherWidget::closeEvent(QCloseEvent* event)
{
    if(m_closeignore) {
        this->slotClose();
        if(m_closeignore) {
            event->ignore();
        } else {
            event->accept();
        }
    } else {
        event->accept();
    }
    m_count = 0;
}

/*
 * public slot
 */
void SbtpMotherWidget::slotSetMaxConnection(int maxconnections)
{
    m_maxconn = maxconnections;
}

/*
 * public slot
 */
void SbtpMotherWidget::slotAddChild(int socket)
{
    m_childs++;
    QString childtext = QString("Count: %1").arg( m_childs, 4, 10 );
    ChildCountLabel->setText(childtext);
    bool full;
    if(m_childs > m_maxconn) {
        full = true;
        g_messenger->slotLogEntry(LogEntry::StatusWARNING,
                QString("SERVER FULL: Sbtp Port %1, max. Connections %2")
                .arg(m_port).arg(m_maxconn));
    } else {
        full = false;
    }
    emit signalNeedChild(socket, full, this);
}

/*
 * public slot
 */
void SbtpMotherWidget::slotRemoveChild()
{
    m_childs--;
    QString childtext = QString("Count: %1").arg( m_childs, 4, 10 );
    ChildCountLabel->setText(childtext);
}


