#include <qlabel.h>
#include <qsocket.h>
#include <qtextstream.h>
#include <qtimer.h>

#include "global.h"
#include "messenger.h"
#include "blmfile.h"

/*
 * The TelnetChildSocket class provides a socket that is connected with a client.
 * For every client that connects to the server, the server creates a new
 * instance of this class.
 */
class TelnetChildSocket : public QSocket
{
    Q_OBJECT

public:
    TelnetChildSocket( int sock, bool repaint, bool full,
            QObject *parent=0, const char *name=0 ) :
    QSocket( parent, name )
    {
        connect( this, SIGNAL(readyRead()),
            SLOT(closeClient()) );
        connect( this, SIGNAL(connectionClosed()),
            SLOT(deleteLater()) );
        connect( this, SIGNAL(connectionClosed()),
            this, SLOT(slotEmitClose()) );
        connect( this, SIGNAL(error(int)),
            this, SLOT(slotEmitClose()) );
        setSocket( sock );

        m_repaintall = repaint;
        m_firstpaint = true;

        if(full) {
            showFull();
            QTimer* timer = new QTimer(this);
            connect(timer, SIGNAL(timeout()), this, SLOT(closeClient()));
            timer->start(100, true);
        } else {
            clearScreen();
            connect(g_messenger, SIGNAL(signalBlmFrame(BlmFrame&)),
                    this, SLOT(slotShowBlmFrame(BlmFrame&)));
        }


    }

    ~TelnetChildSocket()
    {
        emit signalDestruction();
    }

public slots:

    void slotEmitClose() {
        emit signalClosing();
    }

    void slotShowBlmFrame(BlmFrame& frame) {
        QTextStream ts( this );
        ts << "[H";

        if(m_repaintall || m_firstpaint) {
            ts << "/|==================|\\\n";
        } else {
            ts << "\n";
        }
        
        for(int y = 0; y < 8; y++) {
            ts << "||";
            for(int x = 0; x < 18; x++) {
                if(frame.pixel[x][y]) {
                    ts << "#";
                } else {
                    ts << " ";
                }
            }
            ts << "||\n";
        }

        if(m_repaintall || m_firstpaint) {
            ts << "||==================||\n";
            ts << "||                  ||\n";
            ts << "|| HAUS DES LEHRERS ||\n";
            ts << "||                  ||\n";
            ts << "|| Blinken       _  ||\n";
            ts << "||    lights    | | ||\n";
            ts << "||              | | ||\n";
            ts << "======================\n";
            ts << "\n";
            ts << "Press ENTER to finnish\n";

            m_firstpaint = false;
        }

        flush();

    }

    void slotSetRepaintAll(bool repaint) {
        m_repaintall = repaint;
        m_firstpaint = true;
    }

private:
    void clearScreen() {
        QTextStream ts( this );
        ts << "[H\n";
        
        for(int x=0; x < 120; x++) {
            for(int y=0; y < 3; y++) {
                ts << "                                       ";
            }
            ts << "\n";
        }
        ts << "[H\n";
    }

    void showFull() {
        QTextStream ts( this );
        ts << "Sorry, server is full, try again later!\n";
        emit signalClosing();
        this->close();
    }

    bool m_repaintall;
    bool m_firstpaint;


signals:
    void signalClosing();
    void signalDestruction();


private slots:
    void closeClient()
    {
        QTextStream ts( this );
            emit signalClosing();
            this->close();
    }
};
