#include <qlabel.h>
#include "telnetchildwidget.h"
#include "telnetmotherwidget.h"
#include "telnetchildsocket.h"

#include "global.h"
#include "messenger.h"
#include "blmfile.h"

/* 
 *  Constructs a TelnetChildWidget which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 */
TelnetChildWidget::TelnetChildWidget(  int socket, bool repaint, bool full,
        TelnetMotherWidget* serverparent,
        QWidget* parent,  const char* name, WFlags fl )
    : TelnetChildWidgetBase( parent, name, fl )
{
    PortLabel->setText(QString("Port: %1").arg(serverparent->getPort()));
    connect(serverparent, SIGNAL(signalCloseAll()),
            this, SLOT(slotClose()));
    connect(serverparent, SIGNAL(signalAlwaysRepaint(bool)),
            this, SLOT(slotSetAlwaysRepaint(bool)));

    m_child = new TelnetChildSocket(socket, repaint, full, this);
    connect( m_child, SIGNAL(signalClosing()),
                this, SLOT(slotRemoveChild()) );
    connect( m_child, SIGNAL(signalDestruction()),
                this, SLOT(slotDestruction()) );
        
    connect(this, SIGNAL(signalRemoveChild()),
            serverparent, SLOT(slotRemoveChild()));

    m_count = 0;
    m_closesent = false;
    connect(g_messenger, SIGNAL(signalBlmFrame(BlmFrame&)),
                this, SLOT(slotShowFrame()));

}

/*  
 *  Destroys the object and frees any allocated resources
 */
TelnetChildWidget::~TelnetChildWidget()
{
    if(m_child) {
        delete m_child;
    }
}

/*
 * public slot
 */
void TelnetChildWidget::slotClose()
{
    if(!m_closesent) {
        m_closesent = true;
        emit signalRemoveChild();
    }
    close();
}

/*
 * public slot
 */
void TelnetChildWidget::slotShowFrame()
{
    m_count++;
    QString count = QString("Frames: %1").arg( m_count, 8, 10 );
    FrameCountLabel->setText(count);
}

/*
 * public slot
 */
void TelnetChildWidget::slotSetAlwaysRepaint(bool repaint)
{
    m_child->slotSetRepaintAll(repaint);
}

/*
 * public slot
 */
void TelnetChildWidget::slotRemoveChild()
{
    slotClose();
}

void TelnetChildWidget::slotDestruction()
{
    m_child = 0;
    slotRemoveChild();
}


