#include <qevent.h>
#include <qspinbox.h>
#include <qcheckbox.h>
#include <qlabel.h>
#include <qworkspace.h>

#include "telnetmotherwidget.h"
#include "global.h"
#include "messenger.h"
#include "telnetmothersocket.h"

/* 
 *  Constructs a TelnetMotherWidget which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 */
TelnetMotherWidget::TelnetMotherWidget(  int port, int maxconnections, 
        bool alwaysrepaint,
        QWidget* parent,  const char* name, WFlags fl )
    : TelnetMotherWidgetBase( parent, name, fl )
{

    m_repaint = alwaysrepaint;
    telnetFullRepaintBox->setChecked(m_repaint);

    m_port = port;
    QString porttext = QString("Port: %1").arg( m_port, 5, 10 );
    PortLabel->setText(porttext);

    m_maxconn = maxconnections;
    MaxConnectionsBox->setValue(m_maxconn);

    g_messenger->slotLogEntry(LogEntry::StatusINFO,
            QString("Starting Telnet-Server Port %1, max. Connections %2")
            .arg(m_port).arg(m_maxconn));

    connect(g_messenger, SIGNAL(signalBlmFrame(BlmFrame&)),
            this, SLOT(slotShowFrame()));

    g_messenger->slotLogEntry(LogEntry::StatusINFO, "Telnet-Server is running");

    m_socket = new TelnetMotherSocket(m_port, this);
    connect(m_socket, SIGNAL(signalAddChild(int)), 
            this, SLOT(slotAddChild(int)));
    connect(m_socket, SIGNAL(signalRemoveChild()), 
            this, SLOT(slotRemoveChild()));

    m_closeignore = true;
    m_count = 0;
    m_childs = 0;

}

/*  
 *  Destroys the object and frees any allocated resources
 */
TelnetMotherWidget::~TelnetMotherWidget()
{
    // no need to delete child widgets, Qt does it all for us
}

/*
 * public slot
 */
void TelnetMotherWidget::slotClose()
{
    g_messenger->slotLogEntry(LogEntry::StatusINFO,
            QString("Closing Telnet-Server Port %1").arg(m_port));
    emit signalCloseAll();
    delete m_socket;
    m_closeignore = false;
    close();
}

void TelnetMotherWidget::closeEvent(QCloseEvent* event)
{
    if(m_closeignore) {
        this->slotClose();
        if(m_closeignore) {
            event->ignore();
        } else {
            event->accept();
        }
    } else {
        event->accept();
    }
    m_count = 0;
}

/*
 * public slot
 */
void TelnetMotherWidget::slotShowFrame()
{
    m_count++;
    QString count = QString("Frames: %1").arg( m_count, 8, 10 );
    FrameCountLabel->setText(count);
}

/*
 * public slot
 */
void TelnetMotherWidget::slotSetMaxConnection(int maxconnections)
{
    m_maxconn = maxconnections;
}

/*
 * public slot
 */
void TelnetMotherWidget::slotSetAlwaysRepaint(bool repaint)
{
    m_repaint = repaint;
    emit signalAlwaysRepaint(repaint);
}

/*
 * public slot
 */
void TelnetMotherWidget::slotAddChild(int socket)
{
    m_childs++;
    QString childtext = QString("Count: %1").arg( m_childs, 4, 10 );
    ChildCountLabel->setText(childtext);
    bool full;
    if(m_childs > m_maxconn) {
        full = true;
        g_messenger->slotLogEntry(LogEntry::StatusWARNING,
                QString("SERVER FULL: Telnet Port %1, max. Connections %2")
                .arg(m_port).arg(m_maxconn));
    } else {
        full = false;
    }
    emit signalNeedChild(socket, m_repaint, full, this);
}

/*
 * public slot
 */
void TelnetMotherWidget::slotRemoveChild()
{
    m_childs--;
    QString childtext = QString("Count: %1").arg( m_childs, 4, 10 );
    ChildCountLabel->setText(childtext);
}


