#ifndef UDPDYNCONNECTION_H
#define UDPDYNCONNECTION_H
#include <qhostaddress.h>
#include <qsocketdevice.h>
#include <qtimer.h>
#include <qasciidict.h>

#include "udpdynconnectionbase.h"
#include "global.h"
#include "messenger.h"

class BlmFrame;
class QCloseEvent;
class QString;
class QSocketDevice;
class QSocketNotifier;

class UdpDynDict : public QObject
{
    Q_OBJECT

public:
    UdpDynDict(QHostAddress hostaddress, int port,
            QSocketDevice* socket) {
        m_hostaddress = hostaddress;
        m_port = port;
        m_socket = socket;

        m_timer = new QTimer(this);
        
        connect(m_timer, SIGNAL(timeout()),
                this, SLOT(slotTimeOut()));

        m_timer->start(15000, true);

        connect(g_messenger, SIGNAL(signalBlmFrame(BlmFrame&)),
                this, SLOT(slotShowFrame(BlmFrame&)));
    };

    ~UdpDynDict() {
        delete m_timer;
    };

public slots:
    void slotRefresh() {
        m_timer->start(15000, true);
    }

private slots:
    void slotTimeOut() {
        emit signalTimeOut(m_hostaddress, m_port);
    }

    void slotShowFrame(BlmFrame& frame) {
        char Buffer[156];
        // Magic
        Buffer[0] = (char)0xDE;
        Buffer[1] = (char)0xAD;
        Buffer[2] = (char)0xBE;
        Buffer[3] = (char)0xEF;

        // Unknown
        Buffer[4] = (char)0x00;
        Buffer[5] = (char)0x00;
        Buffer[6] = (char)0x00;
        Buffer[7] = (char)0x00;

        // Size
        Buffer[8] = (char)0x00;
        Buffer[9] = (char)BLMWIDTH;
        Buffer[10] = (char)0x00;
        Buffer[11] = (char)BLMHEIGHT;

        char* bufpoint = &Buffer[12];

        for(int height = 0; height < BLMHEIGHT; height++) {
            for(int width = 0; width < BLMWIDTH; width++) {
                if(frame.pixel[width][height] == false) {
                    *bufpoint = (char)0x00;
                } else {
                    *bufpoint = (char)0xFF;
                }
                bufpoint++;
            }
        }

        if(!m_socket->writeBlock(Buffer, 156,
                m_hostaddress, m_port) == -1) {
            qDebug("UDPDYN: Could not write bytes");
        };
    }

signals:
    void signalTimeOut(QHostAddress,int);

private:
    QHostAddress m_hostaddress;
    int m_port;
    QSocketDevice* m_socket;
    QTimer* m_timer;
};
        


class UdpDynConnection : public udpdynconnectionbase
{
    Q_OBJECT

public:
    UdpDynConnection(int port, int maxconn, 
            QWidget* parent = 0, const char* name = 0, WFlags fl = 0 );
    ~UdpDynConnection();

public slots:
    void slotShowFrame();
    void slotClose();
    void slotSocketEvent(int);
    void slotTimeOut(QHostAddress, int);

protected:
    void closeEvent(QCloseEvent* event);

private:
    long m_count;
    int m_conn;
    bool m_closeignore;
    int m_port;
    QSocketDevice* m_socket;
    QSocketNotifier* m_notifier;
    QHostAddress m_hostaddress;
    typedef QAsciiDict<UdpDynDict> MyDict;
    MyDict* m_dict;
};

#endif // UDPDYNCONNECTION_H
