#include <qlabel.h>
#include <qevent.h>
#include <qsocketdevice.h>
#include <qvaluelist.h>
#include <qdns.h>

#include "udpfixconnection.h"
#include "global.h"
#include "messenger.h"

/* 
 *  Constructs a UdpFixConnection which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 */
UdpFixConnection::UdpFixConnection(QString host, int port,
        QWidget* parent,  const char* name, WFlags fl )
    : udpfixconnectionbase( parent, name, fl )
{
    m_count = 0;
    m_host = host;
    m_port = port;

    HostnameLabel->setText(QString("HOST: %1:%2").arg(m_host).arg(m_port));
    
    g_messenger->slotLogEntry(LogEntry::StatusINFO, 
            QString("Starting new UdpFix-Connection to Host %1")
            .arg(m_host));

    m_socket = new QSocketDevice(QSocketDevice::Datagram);
    m_dns = new QDns(m_host);
    connect(m_dns, SIGNAL(resultsReady()), 
            this, SLOT(slotDnsReady()));

    m_closeignore = true;
}

/*  
 *  Destroys the object and frees any allocated resources
 */
UdpFixConnection::~UdpFixConnection()
{
    delete m_socket;
}

/*
 * public slot
 */
void UdpFixConnection::slotShowFrame(BlmFrame& frame)
{
    m_count++;
    QString count = QString("Frames: %1").arg( m_count, 8, 10 );
    FrameCountLabel->setText(count);
    char Buffer[156];
    // Magic
    Buffer[0] = (char)0xDE;
    Buffer[1] = (char)0xAD;
    Buffer[2] = (char)0xBE;
    Buffer[3] = (char)0xEF;

    // Unknown
    Buffer[4] = (char)0x00;
    Buffer[5] = (char)0x00;
    Buffer[6] = (char)0x00;
    Buffer[7] = (char)0x00;

    // Size
    Buffer[8] = (char)0x00;
    Buffer[9] = (char)BLMWIDTH;
    Buffer[10] = (char)0x00;
    Buffer[11] = (char)BLMHEIGHT;

    char* bufpoint = &Buffer[12];

    for(int height = 0; height < BLMHEIGHT; height++) {
        for(int width = 0; width < BLMWIDTH; width++) {
            if(frame.pixel[width][height] == false) {
                *bufpoint = (char)0x00;
            } else {
                *bufpoint = (char)0xFF;
            }
            bufpoint++;
        }
    }

    if(!m_socket->writeBlock(Buffer, 156,
            QHostAddress(m_hostaddress), m_port) == -1) {
        qDebug("UDPFIX: Could not write bytes");
    };
}
/*
 * public slot
 */
void UdpFixConnection::slotDnsReady()
{
    qDebug("Results ready...");
    QValueList<QHostAddress> list = m_dns->addresses();
    QValueList<QHostAddress>::Iterator it = list.begin();
    if( it != list.end() ) {
        m_hostaddress = *it;
        connect(g_messenger, SIGNAL(signalBlmFrame(BlmFrame&)),
                this, SLOT(slotShowFrame(BlmFrame&)));
        IPLabel->setText(QString("IP: %1").arg(m_hostaddress.toString()));
    } else {
        g_messenger->slotLogEntry(LogEntry::StatusWARNING, 
            QString("Could not resolve Hostname %1")
            .arg(m_host));
        close();
    }

    delete m_dns;
}
/*
 * public slot
 */
void UdpFixConnection::slotClose()
{
    g_messenger->slotLogEntry(LogEntry::StatusINFO, "Closing UdpFix-Connection");
    m_closeignore = false;
    close();
}

void UdpFixConnection::closeEvent(QCloseEvent* event)
{
    if(m_closeignore) {
        this->slotClose();
        if(m_closeignore) {
            event->ignore();
        } else {
            event->accept();
        }
    } else {
        event->accept();
    }
}
